#!/bin/sh
#
# TCNV        Startup script for the TC bin.
#
# description: Thin Client Startup script for the TC bin.
#
# processname: tcnv
# config: $STORAGE_PATH/config
# tcfile: $STORAGE_PATH/config/.tcfiles

# Source global variables and function library.

. $TS_GLOBAL


quitNow ()
{
   exit 1
   return 0
}

# The .tcfiles conatains the list of files or directories
# to save. Paths must not be links.

CONFIG_DIR=$STORAGE_PATH/config
CONFIG_FILE=$CONFIG_DIR/.tcfiles
TC_HOME=$STORAGE_PATH/thinstation.profile

echo tcnv config running >> $LOGFILE

if [ ! -d $CONFIG_DIR ]; then
   mkdir $CONFIG_DIR
   if [ $? != 0 ]; then
       echo "Unable to create $CONFIG_DIR directory" >> $LOGFILE
       quitNow
   fi
fi

if [ ! -f "$CONFIG_FILE" ]; then #Create the default config file
   #Backup Home Directory
   echo $TC_HOME >> $CONFIG_FILE
fi
if [ ! -f "$CONFIG_FILE" ]; then
   echo "Unable to create $CONFIG_FILE" >> $LOGFILE
   quitNow
fi

rmcfg() { # Delete tcnv config files
        echo "Removing tcnv Configuration" >> $LOGFILE
        for F in `cat $CONFIG_FILE`
        do
           echo "Deleting $F" >> $LOGFILE
           ( cd /; rm -rf .$F >> $LOGFILE 2>&1 )
        done
        if [ $? != 0 ]; then
           echo "Delete failed, restoring last configuration." >> $LOGFILE
           restore
           return 1
          # quitNow
        fi
        return 0
}

restore() { #Restore tcnv config over TS config
        if [ -f "$CONFIG_DIR/.saved" ]; then
           echo "Restoring tcnv Configuration" >> $LOGFILE
           rmcfg
           if [ $? != 0 ]; then
              echo "Unable to remove home account. Exiting!!" >> $LOGFILE
              quitNow
           fi
           for F in `cat $CONFIG_FILE`
           do
              echo "Copy .$F TO /" >> $LOGFILE
              (cd $CONFIG_DIR; tar cf - .$F) | ( cd /; tar xpf -)
           done
           if [ $? != 0 ]; then
              echo "\nUnable to restore files in $CONFIG_FILE" >> $LOGFILE
              quitNow
           fi
        else
           echo "Save not current, skipping restore." >> $LOGFILE
        fi
        return 0
}

lastcfg() { #Function moves current config files to get ready for a new save
       echo "Saving last tcnv Configuration" >> $LOGFILE

       LASTCFG_DIR=$CONFIG_DIR/lastcfg
       if [ ! -d "$LASTCFG_DIR" ]; then # Create lastcfg if not there
          mkdir $LASTCFG_DIR
       else
          rm -rf $LASTCFG_DIR/* #Remove the old files in lastcfg
       fi

       for F in `cat $CONFIG_FILE`
       do
         echo "Copy .$F TO last config" >> $LOGFILE
         ( cd $CONFIG_DIR; tar cf - .$F ) | ( cd $LASTCFG_DIR; tar xpf -)
        if [ $? != 0 ]; then
            echo "Not Deleting files." >> $LOGFILE
         else
            echo "Removing files." >> $LOGFILE
           ( cd $CONFIG_DIR; rm -rf .$F )
           rm -rf CONFIG_DIR/.saved
         fi
       done
       return 0
}

save() { # Function Saves config file
        echo "Saving tcnv Configuration" >> $LOGFILE

        if [ -f "$CONFIG_DIR/.saved" ]; then
           lastcfg #move the current save files to lastcfg
        else
           echo "Save not current, skipping save of last config." >> $LOGFILE
        fi
        for F in `cat $CONFIG_FILE`
        do
           if [ ! -h $F ]; then
              tar cf - $F | ( cd $CONFIG_DIR; tar xpf -)
           else
              echo "Copying of links is not supported" >> $LOGFILE
           fi
        done
        if [ $? != 0 ]; then
           echo "\nUnable to save files in $CONFIG_FILE" >> $LOGFILE
           quitNow
        else
           touch $CONFIG_DIR/.saved
           echo "Save Complete!" >> $LOGFILE
           return 0
        fi
}

savepkg() { # Function save initial package list
        
        if [ ! -f "$CONFIG_DIR/.fdpkglst" ] ; then
           echo "Saving FD package names." >> $LOGFILE
           ls $STORAGE_PATH/$PKG_PREFIX > $CONFIG_DIR/.fdpkglst
        fi
}

fdpkg() { # Function remove any added packages not in .fdpkglst & lib dir.

        if [ -f "$CONFIG_DIR/.fdpkglst" ] ; then
           for cur_pkg in `ls $STORAGE_PATH/$PKG_PREFIX` ; do
              found=no
              for fd_pkg in `cat $CONFIG_DIR/.fdpkglst` ; do
                 if [ "_$cur_pkg" = "_$fd_pkg" ] ; then 
                    found=yes
                 fi
              done
              if [ $found = 'no' ] ; then
                 echo "Removing package $cur_pkg." >> $LOGFILE
                 rm -rf $STORAGE_PATH/$PKG_PREFIX/$cur_pkg
              fi
            done
            if [ -d "$STORAGE_PATH/lib" ] ; then
               rm -rf "$STORAGE_PATH/lib"
               echo "Removing lib dmc components for TDM installed packages." >> $LOGFILE
            fi
         fi
}
rmdmc() { # Function to remove .dmc directory from FD saved files.

        back=`pwd`
        cd $CONFIG_DIR/fd
        i=1
        while [ $i -lt 8 ] ; do
           if dmc=`ls -a|grep ".dmc"` ; then
              rm -rf .dmc
              if [ $? != 0 ] ; then
                 echo "Unable to delete the .dmc directory. " >> $LOGFILE
              else
                 echo "Directory .dmc deleted." >> $LOGFILE
              fi
              break
           fi
           if [ -d "`ls`" ] ; then
              cd `ls`
           else
              i=8
              break
           fi
           let i++
        done
        if [ $i -eq 8 ] ; then
           echo "The .dmc director is not present." >> $LOGFILE
        fi
        cd $back
        return 0
}     

savefd() { # Function save Factory Default Settings

        #This is done only once after the first boot after a recovery.
        if [ ! -d "$CONFIG_DIR/fd" ] ; then
           echo "Saving tcnv Factory Default  Configuration" >> $LOGFILE
           if [ ! -f $STORAGE_PATH/fd.cfg ] ; then
              cp $STORAGE_PATH/$TC_CONFIG $STORAGE_PATH/fd.cfg
           fi
           savepkg
           mkdir "$CONFIG_DIR/fd"
           for F in `cat $CONFIG_FILE`
           do
              if [ ! -h $F ]; then
                 tar cf - $F | ( cd $CONFIG_DIR/fd; tar xpf -)
              else
                 echo "Copying of links is not supported" >> $LOGFILE
              fi
           done
           if [ $? != 0 ]; then
              echo "\nUnable to save files in $CONFIG_FILE" >> $LOGFILE
              quitNow
           else
              #remove .dmc directory form fd saved files.
              rmdmc
              echo "Save of Factory Default Complete!" >> $LOGFILE
           fi
        else 
           echo "Save of Factory Default not needed." >> $LOGFILE
        fi
           return 0
}

killdaemons() {
  echo "Terminating device manager daemon" >> $LOGFILE
  for pid in `pidof dmc`; do
	kill $pid;
    sleep 1;
    [ -n `pidof dmc` ] && kill -9 `pidof dmc`
  done
}

fd() { # Restore Factory Defaults
	if [ -d "$CONFIG_DIR/fd" ] ; then
        	if [ -f $STORAGE_PATH/fd.cfg ] ; then
           		fdpkg
      	   		killdaemons
           		echo "Restoring Factory Default Configuration" >> $LOGFILE
           		rm -rf $STORAGE_PATH/$TC_CONFIG
           		cp $STORAGE_PATH/fd.cfg $STORAGE_PATH/$TC_CONFIG
           		rmcfg
           		if [ $? != 0 ]; then
              			echo "Unable to remove home account. Exiting!!" >> $LOGFILE
              			quitNow
           		fi
           		for F in `cat $CONFIG_FILE`
           		do
              			echo "Copy .$F TO /" >> $LOGFILE
              			(cd $CONFIG_DIR/fd; tar cf - .$F) | ( cd /; tar xpf -)
           		done
           		if [ $? != 0 ]; then
              			echo "\nUnable to restore Factory Defaults." >> $LOGFILE
              			quitNow
           		fi
           		echo "Factory Defaults restored, rebooting..." >> $LOGFILE
           		reboot
           		# wait until system restarts or hang the whole thing if it goes comlpetely wrong
           		sleep 99999
          	else
            		echo "Factory Default file missing, skipping restore." >> $LOGFILE
          	fi
        else
           	echo "Factory Default directory missing, skipping restore." >> $LOGFILE
        fi
        return 0
}


# If no arugements passed perform a restore and savefd.

# See how we were called
case "$1" in
  restore|"")
        #Run restore & savefd if no arguments are passed.
        savefd
        restore
        ;;
  save)
        save
        ;;
  savefd)
        #This is done only once after the first build
        savefd
        ;;
  fd)   
        #Restore Factory Defaults
        fd
        ##/bin/reboot
        ;;
  *)
        echo $"Usage: $prog {restore|save|savefd|fd|help}"
        quitNow
esac

quitNow

